//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+    FILE : stj_data_cache.c
//+ SUMMARY : SVFtoJSVF buffer cache function
//+     REV : 1.00 (2005/01/21) 
//+  AUTHOR : M.Nakai (Atmark-Techno,Inc.)
//+    MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//

//+<< INCLUDE >>+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
#include <stdio.h>
#include <string.h>
#ifdef WIN32
#include <io.h>
#else //WIN32
#include <unistd.h>
#endif //WIN32
#include <sys/types.h>
 
#include "stj_data_cache.h"

static int _DC_Fd;
static DC_Position _DC_Pos;
static char _DC_Buf[DC_BUF_SIZE];

//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : DataCacheGetData
//+   NOTE : 
//+  PARAM : 
//+ RETURN : "0":SUCCESS,"-1":ERROR
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
static int DataCacheGetData(void){
  _DC_Pos.Ptr=0;
  _DC_Pos.FilePoint=lseek(_DC_Fd,0,SEEK_CUR);
  _DC_Pos.DataSize=read(_DC_Fd,_DC_Buf,DC_BUF_SIZE);
  _DC_Pos.Count=_DC_Pos.DataSize;
  if(_DC_Pos.DataSize<=0){
    return -1;
  }
  return 0;
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : DataCacheReadByte
//+   NOTE : 
//+  PARAM : 
//+ RETURN : "0":SUCCESS,"-1":ERROR
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
int DataCacheReadByte(char *buf) {
  if(_DC_Pos.Count==0){
    if(DataCacheGetData()==-1){
      return -1;
    }
  }
  buf[0]=_DC_Buf[_DC_Pos.Ptr];
  _DC_Pos.Ptr++;
  _DC_Pos.Count--;
  return 0;
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : DataCacheGetDataPrevious
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
static int DataCacheGetDataPrevious(void){
  long Cur;
  long End;

  if(_DC_Pos.FilePoint==0){
    lseek(_DC_Fd,0,SEEK_SET);
    DataCacheGetData();
    return -1;
  }
  
  Cur=lseek(_DC_Fd,0,SEEK_CUR);
  End=lseek(_DC_Fd,0,SEEK_END);
  if(Cur==End && End==_DC_Pos.FilePoint){
    int ReqSize=_DC_Pos.FilePoint%DC_BUF_SIZE;
    ReqSize=((ReqSize)?ReqSize:DC_BUF_SIZE);
    _DC_Pos.FilePoint=lseek(_DC_Fd,(-1)*(ReqSize),SEEK_CUR);
    _DC_Pos.DataSize=read(_DC_Fd,_DC_Buf,ReqSize);
  }else{
    lseek(_DC_Fd,_DC_Pos.FilePoint,SEEK_SET);
    _DC_Pos.FilePoint=lseek(_DC_Fd,(-1)*DC_BUF_SIZE,SEEK_CUR);
    _DC_Pos.DataSize=read(_DC_Fd,_DC_Buf,DC_BUF_SIZE);
  }
  
  _DC_Pos.Count=0;
  if(_DC_Pos.DataSize<=0) return -1;
  _DC_Pos.Ptr=_DC_Pos.DataSize;
  return 0;
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : DataCacheReadBytePrevious
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
int DataCacheReadBytePrevious(char *buf){
  if(_DC_Pos.Ptr==0){
    if(DataCacheGetDataPrevious()==-1){
      return -1;
    }
  }
  _DC_Pos.Count++;
  _DC_Pos.Ptr--;
  buf[0]=_DC_Buf[_DC_Pos.Ptr];
  return 0;
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : DataCachePeekReadByte
//+   NOTE : 
//+  PARAM : 
//+ RETURN : "0":SUCCESS,"-1":ERROR
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
int DataCachePeekReadByte(char *buf){
  int ReadLen;
  if(_DC_Pos.Count==0){
    if((ReadLen=read(_DC_Fd,buf,1))==0){
      return -1;
    }else{
      lseek(_DC_Fd,-1,SEEK_CUR);
      return 0;
    }
  }else{
    buf[0]=_DC_Buf[_DC_Pos.Ptr];
    return 0;
  }
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : DataCacheInit
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
void DataCacheInit(int fd){
  _DC_Fd            =fd;
  memset(_DC_Buf,0,DC_BUF_SIZE);
  if(DataCacheGetData()==-1){
    ;
  }
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : DataCacheSavePosition
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
int DataCacheSavePosition(DC_Position *pos){
  pos->FilePoint =_DC_Pos.FilePoint;
  pos->DataSize  =_DC_Pos.DataSize;
  pos->Ptr       =_DC_Pos.Ptr;
  pos->Count     =_DC_Pos.Count;
  return 0;
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : DataCacheLoadPosition
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
int DataCacheLoadPosition(DC_Position *pos){
  if(_DC_Pos.FilePoint!=pos->FilePoint || _DC_Pos.DataSize!=pos->DataSize){
    lseek(_DC_Fd,pos->FilePoint,SEEK_SET);
    DataCacheGetData();
  }
  _DC_Pos.Ptr   =pos->Ptr;
  _DC_Pos.Count =pos->Count;
  return 0;
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : DataCacheLoadPositionPrevious
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
int DataCacheLoadPositionPrevious(DC_Position *pos){
  if(_DC_Pos.FilePoint!=pos->FilePoint || _DC_Pos.DataSize!=pos->DataSize){
    _DC_Pos.FilePoint=lseek(_DC_Fd,pos->FilePoint,SEEK_SET);
    DataCacheGetData();
  }
  _DC_Pos.Ptr   =pos->Ptr;
  _DC_Pos.Count =pos->Count;
  return 0;
}
