//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+    FILE : stj_shift.c
//+ SUMMARY : SVFtoJSVF bit shift function
//+     REV : 1.00 (2005/01/21) 
//+  AUTHOR : M.Nakai (Atmark-Techno,Inc.)
//+    MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//

//+<< INCLUDE >>+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
#include <stdio.h>
#include <string.h>

#ifdef WIN32
#define inline __inline
#endif //WIN32

//---------------------------------------------------------------------------//

//#define __SHIFT_DEBUG "Shift Debug Message enable"

#define Bit2ByteLen(bitlen) (bitlen/8+(bitlen%8?1:0))

inline unsigned char L_S(const unsigned char data,const int sh){
  return ((data<<sh)&(~( (1<<(sh))-1)));
}

inline unsigned char R_S(const unsigned char data,const int sh){
  return ((data>>sh)&((1<<(8-sh))-1));
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
static void CopyData(unsigned char *dst,unsigned char *src,
		     int dst_size,int src_size,
		     int dst_offset,int src_offset,
		     int copy_size){
  int i;
  int loop=Bit2ByteLen(copy_size);

  int l_shift=dst_offset%8;
  int r_shift=8-l_shift;
  
  int dst_ptr=Bit2ByteLen(dst_size)-1 - dst_offset/8;
  int src_ptr=Bit2ByteLen(src_size)-1 + src_offset/8;

  unsigned char last_mask=((1<<((copy_size%8)?copy_size%8:(copy_size)?8:0))-1);

  if(src_offset%8){
    unsigned char tmp=src[src_ptr]&~((1<<(src_offset%8))-1);
    unsigned char tmp2=R_S(tmp,src_offset%8);
    int cur_dst_ptr=dst_ptr;

    if(src_offset%8>copy_size){
      tmp2&=(1<<copy_size)-1;
    }

    copy_size-=8-(src_offset%8);
    if(dst_offset%8){
      if(dst_offset%8+src_offset%8>8){
	int tl_shift=8-(dst_offset%8);
	int tr_shift=(dst_offset+src_offset)%8;

	dst[cur_dst_ptr--] |= L_S(tmp2   ,tl_shift);
	if(cur_dst_ptr>=0){
	  dst[cur_dst_ptr]    = R_S(tmp2   ,tr_shift);
	}
	dst_offset += 8-(src_offset%8);
	src_offset=0;
      }else{
	dst[cur_dst_ptr] |= L_S(tmp2   ,dst_offset);
	
	dst_offset += 8-(src_offset%8);
	src_offset=0;	  
      }
    }else{
      dst[cur_dst_ptr]=tmp2;
      dst_offset +=8-(src_offset%8);
      src_offset=0;
    }
    if(src_ptr>0){
      src_ptr--;
    }else{
      return;
    }
    l_shift=dst_offset%8;
    r_shift=8-l_shift;
    loop=Bit2ByteLen(copy_size);
    last_mask=((1<<((copy_size%8)?copy_size%8:(copy_size)?8:0))-1);
  }

  if(dst_offset%8){
    for(i=0;i<loop-1;i++){
      dst[dst_ptr--] |= L_S(src[src_ptr]   ,l_shift);
      if(dst_ptr>=0){
	dst[dst_ptr]  = R_S(src[src_ptr--] ,r_shift);
      }
    }
    dst[dst_ptr--] |= L_S((unsigned char)(src[src_ptr] & last_mask),l_shift);
    if(dst_ptr>=0){
      dst[dst_ptr]  = R_S((unsigned char)(src[src_ptr--] & last_mask),r_shift);
    }
  }else{
    for(i=0;i<loop-1;i++){
      dst[dst_ptr--]=src[src_ptr--];
    }
    dst[dst_ptr--]=(src[src_ptr--]&last_mask);
  }
  return;
}
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+   FUNC : 
//+   NOTE : 
//+  PARAM : 
//+ RETURN : 
//+   MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
int RequestData(unsigned char *dst,unsigned char *src,
		const int dst_size,const int src_size,
		const int dst_offset,const int src_offset,
		const int index,const int threshold){
  int WishSize;
  int ReqSize=0;

  if(src_size==0){
    return 0;
  }
  if(index<src_size+threshold && dst_offset<dst_size){
    WishSize=src_size;
    if(dst_size-dst_offset<WishSize){
      ReqSize=dst_size-dst_offset;
    }else{
      ReqSize=WishSize;
    }
    CopyData(dst,src,dst_size,src_size,dst_offset,src_offset,ReqSize);
  } 
  return ReqSize;
}
